'use strict';

var _execa = _interopRequireDefault(require("execa"));

var _fs = _interopRequireDefault(require("fs"));

var _exec = _interopRequireDefault(require("../../utils/exec"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/**
 * Install Nuxt.js modules
 *
 * @param {String[]} deps - Nuxt.js modules
 * @returns {Promise<void>}
 */
var installDeps = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(deps) {
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return (0, _exec["default"])("npm install --save ".concat(deps), 'Installing Nuxt.js modules', "".concat(deps, " is successfully installed and configured"), {
              cwd: 'client'
            });

          case 2:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function installDeps(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Serve the webapp locally
 *
 * @param {String} projectConfig - Boilerplate template of choice
 * @param {String} templateDir - The directory to execute shell command (client/server)
 * @returns {Promise<void>}
 */


var serveProject = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(projectConfig, templateDir) {
    var port, projectTemplate, modules, isConfigured, installCandidate, modulesWithPrefix, deps, cmd;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            projectTemplate = projectConfig.template;

            if (templateDir === 'client') {
              port = projectTemplate === 'Nuxt.js' ? '3000' : '3002';
            } else {
              port = projectTemplate === 'GraphQL' ? '9000/graphql' : '9000/api';
            }

            if (_fs["default"].existsSync("./".concat(templateDir, "/node_modules"))) {
              _context2.next = 5;
              break;
            }

            _context2.next = 5;
            return (0, _exec["default"])('npm install', 'Installing dependencies in the background. Hold on...', 'Dependencies were successfully installed', {
              cwd: templateDir
            });

          case 5:
            if (!(templateDir === 'client' && projectTemplate === 'Nuxt.js')) {
              _context2.next = 24;
              break;
            }

            modules = projectConfig.modules, isConfigured = projectConfig.isConfigured;

            if (isConfigured) {
              _context2.next = 24;
              break;
            }

            installCandidate = modules.filter(function (module) {
              return ['pwa', 'axios', 'content'].includes(module);
            }); // Do not proceed if the user haven't opted for any Nuxt.js modules

            if (!installCandidate.length) {
              _context2.next = 22;
              break;
            }

            // Add the @nuxtjs prefix
            modulesWithPrefix = installCandidate.map(function (module) {
              return "".concat(module === 'content' ? "@nuxt/".concat(module) : '@nuxtjs', "/").concat(module);
            } // @nuxt/content has different prefix
            ); // @nuxtjs/pwa is to be installed as a devDependency

            if (!installCandidate.includes('pwa')) {
              _context2.next = 20;
              break;
            }

            _context2.next = 14;
            return (0, _exec["default"])('npm install --save-dev @nuxtjs/pwa', 'Installing Nuxt.js pwa module', '@nuxtjs/pwa is successfully installed and configured', {
              cwd: templateDir
            });

          case 14:
            if (!(installCandidate.length > 1)) {
              _context2.next = 18;
              break;
            }

            deps = modulesWithPrefix.filter(function (module) {
              return module !== '@nuxtjs/pwa';
            }).join(' ');
            _context2.next = 18;
            return installDeps(deps);

          case 18:
            _context2.next = 22;
            break;

          case 20:
            _context2.next = 22;
            return installDeps(modulesWithPrefix.join(' '));

          case 22:
            // Update the isConfigured key
            projectConfig.isConfigured = true;

            _fs["default"].writeFileSync('.mevnrc', JSON.stringify(projectConfig, null, 2));

          case 24:
            cmd = 'serve';

            if (projectTemplate === 'Nuxt.js') {
              cmd = 'dev';
            }

            _execa["default"].shell("npm run ".concat(cmd, " -- --port ").concat(port, " --open"), {
              stdio: 'inherit',
              cwd: templateDir
            });

          case 27:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function serveProject(_x2, _x3) {
    return _ref2.apply(this, arguments);
  };
}();

module.exports = serveProject;